#include <gtk/gtk.h>
#include <vte/vte.h>
#include <vte/reaper.h>

GtkBuilder *builder;
GtkWidget *mainWindow;
GtkWidget *vte;

gboolean loadGui();
void create_terminal();
void run_shell_in_terminal();
void vte_child_exited(VteReaper *vtereaper, gint arg1, gint arg2, gpointer user_data);
void on_execute_command_click(GtkWidget *widget, gpointer user_data);

gint main(gint argc, gchar *argv[])
{
	gtk_init(&argc, &argv);
    
    if (loadGui()) {
        mainWindow = GTK_WIDGET(gtk_builder_get_object(builder, "mainWindow"));
        GtkWidget *btnExecute = GTK_WIDGET(gtk_builder_get_object(builder, "btnExecute"));
        if (mainWindow && btnExecute) {
            g_signal_connect(G_OBJECT(mainWindow), "destroy", G_CALLBACK(gtk_main_quit), NULL);
            g_signal_connect(G_OBJECT(btnExecute), "clicked", G_CALLBACK(on_execute_command_click), NULL);
            create_terminal();            
            gtk_widget_show_all(mainWindow);
            gtk_main();
        }
    }
    
    if (builder) {
        g_object_unref(builder);
    }
    
	return 0;
}


gboolean loadGui() {
    GError *error;
    
    builder = gtk_builder_new();
	error = NULL;
	gtk_builder_add_from_file(builder, "main_window.ui", &error);
	if (error) {
		g_print("Wystąpił błąd: %s\n", error->message);
		g_error_free(error);
		return FALSE;
	}
    return TRUE;
}

void run_shell_in_terminal() {
    GError *error;        
    char *startterm[2] = {0, 0};
    startterm[0] = vte_get_user_shell();
    
    error = NULL;
    vte_terminal_fork_command_full(VTE_TERMINAL(vte),
           VTE_PTY_DEFAULT, // Tryb uruchomienia shella
           NULL,  // Katalog roboczy; ustawiony na NULL przyjmuje wartość bieżącego katalogu roboczego
           startterm, // Polecenie do wykonania
           NULL, // NULL lub lista zmiennych środowiskowych ustawianych w terminalu
           (GSpawnFlags)(G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH),  // Flagi dla tworzonego procesu
           NULL, // Funkcja konfiguracyjna wywoływana przed wykonaniem funkcji exec
           NULL, // Dane przekazywane do funkcji konfiguracyjnej
           NULL, // NULL albo zmienna przechowująca PID potomka
           &error // Zmienna przechowująca informację o błędzie
           );
    if (error) {
        g_print("Wystąpił błąd: %s\n", error->message);
        g_error_free(error);
    }
    g_free(startterm[0]);
}

void create_terminal() {
    GtkWidget *frame_alignment = GTK_WIDGET(gtk_builder_get_object(builder, "alignment1"));
    if (frame_alignment) {
        vte = vte_terminal_new();
        g_signal_connect(G_OBJECT(vte), "child-exited", G_CALLBACK(vte_child_exited), NULL);
        gtk_container_add(GTK_CONTAINER(frame_alignment), vte);
        
        run_shell_in_terminal();
    }
}

void vte_child_exited(VteReaper *vtereaper, gint arg1, gint arg2, gpointer user_data) {
    // if user exits shell by pressing Ctrl+D or executing "exit" command,
    // run shell again
    run_shell_in_terminal();
}

void on_execute_command_click(GtkWidget *widget, gpointer user_data) {
    GtkWidget *edtCommand = GTK_WIDGET(gtk_builder_get_object(builder, "edtCommand"));
    if (edtCommand) {
        const gchar *command = gtk_entry_get_text(GTK_ENTRY(edtCommand));
        gchar *command_with_newline = g_strdup_printf("%s\r", command);
        
        vte_terminal_feed_child(VTE_TERMINAL(vte), command_with_newline, -1);
        
        g_free(command_with_newline);
    }
}
